// GamePanel.java
//
// LoneStranger's base code for LD48_11
//
// This is derived from JumpingJack code by 
// Andrew Davison, April 2005, ad@fivedots.coe.psu.ac.th
// from the book Killer Game Programming in Java published by 
// O'Reilly.  http://www.oreilly.com/catalog/killergame/
//
// I stripped all the game logic and left the parts for loading 
// images and sounds and the game loop.

package net.lonestranger.game.engine;

import net.lonestranger.common.ClipsLoader;
import net.lonestranger.common.ImagesLoader;
import net.lonestranger.common.ImagesPlayerWatcher;
import net.lonestranger.game.object.Hud;
import net.lonestranger.game.object.Phrase;
import net.lonestranger.game.object.PopupText;
import net.lonestranger.game.object.Wall;
import net.lonestranger.game.object.WordKeeper;


import net.lonestranger.network.Report;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class GamePanel extends JPanel
                     implements Runnable, ImagesPlayerWatcher
{
  private static final int PWIDTH = 800;   // size of panel
  private static final int PHEIGHT = 600;

  //private static final int CAR_OX = 400;
  //private static final int CAR_OY = 560;
  
  // stores the mouse cursor from the last time it was moved.
  private int mouse_x;
  private int mouse_y;
  
  private static final int NO_DELAYS_PER_YIELD = 16;
  /* Number of frames with a delay of 0 ms before the animation thread yields
     to other running threads. */
  private static final int MAX_FRAME_SKIPS = 5;
    // no. of frames that can be skipped in any one animation loop
    // i.e the games state is updated but not rendered

  // image, clips loader information files
  private static final String IMS_INFO = "imsInfo.txt";
  private static final String SNDS_FILE = "clipsInfo.txt";
  private static final String PHRS_FILE = "categories.txt";

  // are we taking a screenshot this time around?
  private volatile boolean screenshot = false;
  
  private Thread animator;           // the thread that performs the animation
  private volatile boolean running = false;   // used to stop the animation threa (and kill the game?)
  private volatile boolean isPaused = false;

  private long period;                // period between drawing in _nanosecs_

  private Game gameTop;
  private ClipsLoader clipsLoader;

  private Wall wallObj;
  private Phrase phraseObj;
  private Hud hudObj;
  private PopupText popupObj;
  private WordKeeper wordKeeperObj;
  
  private long gameStartTime;   // when the game started
  private int timeSpentInGame;	// for overall program execution

  // used at game termination
  private volatile boolean gameOver = false;
  private int score = 0;

  // for displaying messages
  private Font msgsFont;
  private FontMetrics metrics;

  // off-screen rendering
  private Graphics dbg;
  private Image dbImage = null;
  
  // to display the title/help screen
  private boolean showHelp = false;
  private BufferedImage helpIm;

  private int STATE_MENU = 0;
  private int STATE_GAME = 1;
  private int STATE_OVER = 2;
 
  private int CONDITION_NONE = 0;
  private int CONDITION_LOSE = 1;
  private int CONDITION_WIN  = 2;
  
  
  private int state 	= STATE_GAME;
  private int condition = CONDITION_NONE;
  
  private int scorePoints = 0;
  private int scoreChars = 0;
  
  
  private BufferedImage backgroundImg;

  Cursor oldCursor;
  Cursor invisibleCursor;
  
  private Report rep;
//  private Background backgroundObj;
  
  public GamePanel(Game g, long period)
  {
    gameTop = g;
    this.period = period;

    setDoubleBuffered(false);
    setBackground(Color.white);
    setPreferredSize( new Dimension(PWIDTH, PHEIGHT));

    setFocusable(true);
    requestFocus();    // the JPanel now has focus, so receives key events

    addKeyListener( new KeyAdapter() {
       public void keyPressed(KeyEvent e)
       { processKey(e);  }
     });

    addMouseListener( new MouseAdapter() {
        public void mousePressed(MouseEvent me) 
        { 
        	processMousePressed(me); 
        	
        } 
        
        public void mouseReleased(MouseEvent me)
        {
        	processMouseReleased(me);
        }
    });

    addMouseMotionListener( new MouseMotionAdapter() {
        public void mouseMoved(MouseEvent me)
        { 
        	processMouseMoved(me); 
        }

        public void mouseDragged(MouseEvent me)
        { 
        	processMouseMoved(me);
        	processMousePressed(me);
        }
        
    });
    
    
    
    // initialize the loaders
    ImagesLoader imsLoader = new ImagesLoader(IMS_INFO);
    clipsLoader = new ClipsLoader(SNDS_FILE);

    // initialize the game entities here
    backgroundImg = imsLoader.getImage("paper01"); 
    
    wordKeeperObj = new WordKeeper(PHRS_FILE);
    
   // currently just sends a test packet to the remote server.
   // eventually, it should report scores when the game ends.
/* 	try {
		rep = new Report();
		rep.setURL("http://localhost/dataStore/data.php");
		rep.setData("TEST DATA");
		//rep.phoneHome();
		System.out.println(rep.getData());
		rep.setData(rep.getData());
		System.out.println(rep.getData());
		
	} catch (MalformedURLException mue) {
		mue.printStackTrace();
	}
*/

    wallObj = new Wall(0, 0, 10, PHEIGHT, imsLoader, "shredder");
    wallObj.setActive(true);
    wallObj.loopImage(33, 1);
    phraseObj = new Phrase();

    phraseObj.setPhraseSet(wordKeeperObj.getRandomSet());
    phraseObj.randomizePhrase();
    
    popupObj = new PopupText(0, 0, PWIDTH, PHEIGHT, imsLoader, "text_game_over");
    
    hudObj = new Hud();
    
    
    // prepare title/help screen
    helpIm = imsLoader.getImage("title");
    showHelp = true;    // show at start-up?
    isPaused = true;

    // set up message font
    msgsFont = new Font("SansSerif", Font.BOLD, 24);
    metrics = this.getFontMetrics(msgsFont);

  //  this.pauseGame();
        
  }  // end of GamePanel()


  private void processMouseClick(MouseEvent e)
  {
	  // get mouse location
	//  int mx = e.getX();
	//  int my = e.getY(); 
	  
  }

  private void processMousePressed(MouseEvent e)
  {
	  // get mouse location
	  int mx = e.getX();
	  int my = e.getY(); 
	  
	  if (e.getButton() == e.BUTTON1)
	  {
		  //carObj.gasPressed();		  
	  }

	  if (e.getButton() == e.BUTTON3)
	  {
		  //carObj.brakePressed();
		  //if (carObj.getSpeed() > 20)
		  //clipsLoader.play("brake", false);
	  }
	  
	  
  }
  
  private void processMouseReleased(MouseEvent e)
  {
	  // get mouse location
	  int mx = e.getX();
	  int my = e.getY(); 
	  
	  if (e.getButton() == e.BUTTON1)
	  {
		  //carObj.gasReleased();
	  }
	  if (e.getButton() == e.BUTTON3)
	  {  
		  //carObj.brakeReleased();
	  }
  }  
  
  private void processMouseMoved(MouseEvent e)
  {
	// check for mouse over the game area and highlight that cell.
	int mx = e.getX();
	int my = e.getY();
	
	mouse_x = e.getX();
	mouse_y = e.getY();
		
	if (mouse_x > PWIDTH)
		mouse_x = PWIDTH;
	if (mouse_x < 0)
		mouse_x = 0;
  }
  
  private void processKey(KeyEvent e)
  // handles termination, help, and game-play keys
  {
    int keyCode = e.getKeyCode();

    //System.out.println(state);
    if (state == STATE_GAME)
    {
    // termination keys
	   // listen for esc, q, end, ctrl-c on the canvas to
	    // allow a convenient exit from the full screen configuration
	    if ((keyCode == KeyEvent.VK_ESCAPE) || (keyCode == KeyEvent.VK_END) ||
	        ((keyCode == KeyEvent.VK_C) && e.isControlDown()) )
	      running = false;
	
	    // help controls
	    if (keyCode == KeyEvent.VK_F1) {
	      if (showHelp) {  // help being shown
	        showHelp = false;  // switch off
	        isPaused = false;
	      }
	      else {  // help not being shown
	       showHelp = true;    // show it
	       isPaused = true;    // isPaused may already be true
	      }
	    }
	
	    	    
	    
	    // game-play keys
	    if (!isPaused && !gameOver) 
	    {
	      // move the sprite and ribbons based on the arrow key pressed
	      
	    	
	    	if (keyCode >= KeyEvent.VK_A && keyCode <= KeyEvent.VK_Z)
	    	{
				char key = ' ';
				
				int onmask = e.SHIFT_DOWN_MASK;
				int offmask = 0;
				  
				if ((e.getModifiersEx() & (onmask | offmask)) == onmask)
					  key = (char)(keyCode);
				else
					  key = (char)(keyCode + 32);
				 
				int result = phraseObj.checkNextLetter(key);
				
				processResult(result);

	    	}	
	    	else if (keyCode == KeyEvent.VK_SPACE || (keyCode >= KeyEvent.VK_0 && keyCode <= KeyEvent.VK_9) )
	    	{
	    		char key = ' ';
				
				int onmask = e.SHIFT_DOWN_MASK;
				int offmask = 0;
				  
				if ((e.getModifiersEx() & (onmask | offmask)) == onmask)
				{
					switch(keyCode)
					{
						case KeyEvent.VK_1:
							key = '!';
							break;
						case KeyEvent.VK_2:
							key = '@';
							break;
						case KeyEvent.VK_3:
							key = '#';
							break;
						case KeyEvent.VK_4:
							key = '$';
							break;
						case KeyEvent.VK_5:
							key = '%';
							break;
						case KeyEvent.VK_6:
							key = '^';
							break;
						case KeyEvent.VK_7:
							key = '&';
							break;
						case KeyEvent.VK_8:
							key = '*';
							break;
						case KeyEvent.VK_9:
							key = '(';
							break;
						case KeyEvent.VK_0:
							key = ')';
							break;
						default:
							key = '';
					}
					
					//key = (char)(keyCode);
				}
				else
					  key = (char)(keyCode);
				 
				int result = phraseObj.checkNextLetter(key);
	    		processResult(result);
	    		
	    	}
	    	else if (keyCode == KeyEvent.VK_COMMA
	    			|| keyCode == KeyEvent.VK_PERIOD
	    			|| keyCode == KeyEvent.VK_SLASH
	    			|| keyCode == KeyEvent.VK_SEMICOLON
	    			|| keyCode == KeyEvent.VK_QUOTE
	    			|| keyCode == KeyEvent.VK_OPEN_BRACKET
	    			|| keyCode == KeyEvent.VK_CLOSE_BRACKET
	    			|| keyCode == KeyEvent.VK_BACK_SLASH
	    			|| keyCode == KeyEvent.VK_BACK_QUOTE
	    			|| keyCode == KeyEvent.VK_MINUS
	    			|| keyCode == KeyEvent.VK_EQUALS)
	    	{
	    		char key = ' ';
				
				int onmask = e.SHIFT_DOWN_MASK;
				int offmask = 0;
				  
				if ((e.getModifiersEx() & (onmask | offmask)) == onmask)
				{
					switch(keyCode)
					{
						case KeyEvent.VK_COMMA:
							key = '<';
							break;
						case KeyEvent.VK_PERIOD:
							key = '>';
							break;
						case KeyEvent.VK_SLASH:
							key = '?';
							break;
						case KeyEvent.VK_SEMICOLON:
							key = ':';
							break;
						case KeyEvent.VK_QUOTE:
							key = '"';
							break;
						case KeyEvent.VK_OPEN_BRACKET:
							key = '{';
							break;
						case KeyEvent.VK_CLOSE_BRACKET:
							key = '}';
							break;
						case KeyEvent.VK_BACK_SLASH:
							key = '|';
							break;
						case KeyEvent.VK_BACK_QUOTE:
							key = '~';
							break;
						case KeyEvent.VK_MINUS:
							key = '_';
							break;
						case KeyEvent.VK_EQUALS:
							key = '+';
							break;
						default:
							key = '';
					}
					
					
					
					//key = (char)(keyCode);
				}
				else
				{			
					key = (char)(keyCode);
				
					if (keyCode == 222)
						key = '\'';
					if (keyCode == 192)
						key = '`';
				
				}
				 
				int result = phraseObj.checkNextLetter(key);
	    		processResult(result); 	
	    		
	    		
	    	}
	    	else if (keyCode == KeyEvent.VK_ENTER)
	    	{
    			int failedKeys = phraseObj.getFailedKeys();
    			int phraseLength = phraseObj.getPhraseLength();
	    		
	    		int result = phraseObj.checkPhraseDone();
	    		
	    		if (result == 0 || result == 1)
	    		{
	    			clipsLoader.play("complete01", false);

	
	    			int rate = 100 * (phraseLength - failedKeys) / phraseLength;
	    /*    			
	    			if (rate == 100)
	    			{
	    				popupObj.setImage("text_flawless");
	    				popupObj.setTimer(30);
	    			}
	    			else if (rate > 98)
	    			{
	    				popupObj.setImage("text_terrific");
	    				popupObj.setTimer(30);
	    			}
	    			else if (rate > 96)
	    			{
	    				popupObj.setImage("text_great");
	    				popupObj.setTimer(30);
	    			}
	    			else if (rate > 94)
	    			{
	    				popupObj.setImage("text_super");
	    				popupObj.setTimer(30);
	    			}
	*/    			
	    			if (rate == 100)
	    			{
	    				popupObj.setImage("text_ok");
						popupObj.setTimer(30);
						
	    			}
	    			else if (rate < 90)
	    			{
	    				popupObj.setImage("text_bad");
						popupObj.setTimer(30);
	    			}
	    			
	    			
	    			
	    		}
	    		else if (result == -1)
	    		{
	    			clipsLoader.play("notcomplete01", false);
	    		}
	    		
	    		
	    		if (result == 1) // wave completed
	    		{
	    			popupObj.setImage("text_stage_complete");
	    			popupObj.setTimer(30*5);   			
	    			
	    			phraseObj.setPhraseCount(10);

	    			wordKeeperObj.putRandomSet(phraseObj);
	    			String a = phraseObj.getPhraseSetName();
	    			popupObj.setAdditionalText(a.substring(0,a.indexOf(".")));
	    			//phraseObj.setPhraseSet(wordKeeperObj.getRandomSet());
	    			
					//wallObj.speedChange(-.25);
	    			wallObj.setSpeed(.25);
	    		}
	    		
	    	}
			else if (keyCode == KeyEvent.VK_LEFT) {
			 //carObj.moveLeft();	// return t/f on success (hit edge?)
			}
			else if (keyCode == KeyEvent.VK_RIGHT) {
			 //carObj.moveRight();
			}
			else if (keyCode == KeyEvent.VK_UP) {
				  
			}
			else if (keyCode == KeyEvent.VK_DOWN) {
			
			}
	    }
    }
    else if (state == STATE_OVER)
    {
    	System.out.println("state over");
    }
    
    if (!screenshot)
    {
    	if (keyCode == KeyEvent.VK_F10)
    	{
    		screenshot = true;
    	}
    }
        
  }  // end of processKey()


	public void processResult(int result)
	{
		if (result == 0)	// success
		{
			clipsLoader.play("type01", false);
			hudObj.addCharacter();
		}
		else if (result == -1) // fail
		{
			clipsLoader.play("wrongkey01", false);
			hudObj.mistypeCharacter();
			wallObj.speedChange(.25);
		}
	}
  
  public void sequenceEnded(String imageName)
  // called by ImagesPlayer when the animation finishes
  {  
	 //imagePlayer.restartAt(0);   // reset animation for next time


     
  } // end of sequenceEnded()



  public void addNotify()
  // wait for the JPanel to be added to the JFrame before starting
  { super.addNotify();   // creates the peer
    startGame();         // start the thread
  }


  private void startGame()
  // initialise and start the thread
  {
    if (animator == null || !running) {
      animator = new Thread(this);
      animator.start();
    }
    
    BufferedImage cimage = getGraphicsConfiguration().createCompatibleImage(1, 1, Transparency.BITMASK);
    Graphics2D cg = cimage.createGraphics();
    cg.setBackground(new Color(0,0,0,0));
    cg.clearRect(0,0,1,1);
     
    invisibleCursor = this.getToolkit().createCustomCursor(cimage, new Point(0,0), "Invisible");
    //oldCursor = this.getCursor();
    this.setCursor(invisibleCursor);    
    Robot r;
	try {
		r = new Robot();
	    r.mouseMove(PWIDTH/2, PHEIGHT/2);

	} catch (AWTException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
    

  } // end of startGame()


  // ------------- game life cycle methods ------------
  // called by the JFrame's window listener methods


  public void resumeGame()
  // called when the JFrame is activated / deiconified
  { 
	  if (!showHelp)    // CHANGED
	  {
		  isPaused = false;
		  setCursor(invisibleCursor);
	  }
	  
  }

  public void pauseGame()
  // called when the JFrame is deactivated / iconified
  { 
	  isPaused = true;   
	  setCursor(oldCursor);
  }


  public void stopGame()
  // called when the JFrame is closing
  {  
	  running = false;  
	  setCursor(oldCursor);
  }
 
  // ----------------------------------------------

  public void run()
  /* The frames of the animation are drawn inside the while loop. */
  {
    long beforeTime, afterTime, timeDiff, sleepTime;
    long overSleepTime = 0L;
    int noDelays = 0;
    long excess = 0L;

    gameStartTime = System.nanoTime();
    beforeTime = gameStartTime;

	popupObj.setTimer(0);
    
    running = true;

    while(running) {
      gameUpdate();
      gameRender();
      paintScreen();

      afterTime = System.nanoTime();
      timeDiff = afterTime - beforeTime;
      sleepTime = (period - timeDiff) - overSleepTime;

      if (sleepTime > 0) {   // some time left in this cycle
        try {
          Thread.sleep(sleepTime/1000000L);  // nano -> ms
        }
        catch(InterruptedException ex){}
        overSleepTime = (System.nanoTime() - afterTime) - sleepTime;
      }
      else {    // sleepTime <= 0; the frame took longer than the period
        excess -= sleepTime;  // store excess time value
        overSleepTime = 0L;

        if (++noDelays >= NO_DELAYS_PER_YIELD) {
          Thread.yield();   // give another thread a chance to run
          noDelays = 0;
        }
      }

      beforeTime = System.nanoTime();

      /* If frame animation is taking too long, update the game state
         without rendering it, to get the updates/sec nearer to
         the required FPS. */
      int skips = 0;
      while((excess > period) && (skips < MAX_FRAME_SKIPS)) {
        excess -= period;
        gameUpdate();    // update state but don't render
        skips++;
      }
    }
    System.exit(0);   // so window disappears
  } // end of run()


  private void gameUpdate()
  {
    if (!isPaused && !gameOver) {
    	  	
    	if (state == STATE_GAME)
    	{
    		wallObj.update();	
    		phraseObj.update();
    		hudObj.Update();
    		int result = phraseObj.checkWallIntersection(wallObj.getXPosn() + wallObj.getWidth() - 5);
        	if (result == -1)
        	{
        		state = STATE_OVER;
        		condition = CONDITION_LOSE;
        		clipsLoader.play("explosion01", false);
    			popupObj.setImage("text_game_over");
    			popupObj.setTimer(-1);
        	}

    	}
    	
    	popupObj.Update();
    	
//    	carObj.updateMovement(mouse_x, mouse_y);
  	  
//    	double curspeed = carObj.getSpeed();
//    	String sound = carObj.getSound();
    	
/*    	String clip = "";
    	if(curspeed == 0)
    		clip = "motorON-low";
    	else if (curspeed < 50)
    		clip = "motorON-med";
    	else if (curspeed < 80)
    		clip = "motorON-high";
    	else 
    		clip = "motorON-highest";
    	
    	carObj.setSound(clip);
    	
    	if (!clip.equals(sound) && !clip.equals(""))
    	{
    		clipsLoader.stop("motorON-low");
    		clipsLoader.stop("motorON-med");
    		clipsLoader.stop("motorON-high");
    		clipsLoader.stop("motorON-highest");    		
    		clipsLoader.play(clip, true);
    	}
    	else if (clip.equals(""))
    	{
    		clipsLoader.stop("motorON-low");
    		clipsLoader.stop("motorON-med");
    		clipsLoader.stop("motorON-high");
    		clipsLoader.stop("motorON-highest");    		
    	}
    		
    //	carObj.update();
    	
    	backgroundObj.update();
    	
    	roadObj.update();
    	
    	//oncomingObj.update(carObj.getSpeed());
*/    	
    //	objManager.updateScene(carObj.getSpeed());
    	
//    	hudObj.setSpeed(carObj.getSpeed());
    	
    }
  }  // end of gameUpdate()


  private void gameRender()
  {
    if (dbImage == null){
      dbImage = createImage(PWIDTH, PHEIGHT);
      if (dbImage == null) {
        System.out.println("dbImage is null");
        return;
      }
      else
        dbg = dbImage.getGraphics();
    }

    // draw a white background
//    dbg.setColor(Color.black);
//    dbg.fillRect(0, 0, PWIDTH, PHEIGHT);

    // draw the game elements: order is important
    //
    dbg.drawImage(backgroundImg, 0, 0, null);

    wallObj.draw(dbg);
    phraseObj.draw(dbg);
    popupObj.draw(dbg);
    hudObj.draw(dbg);

    
/*	if (state == STATE_OVER)
	{
		if (condition == CONDITION_LOSE)
		{
			String lose = "You Lose!";
			dbg.drawString(lose, PHEIGHT / 2, (PWIDTH/2)- (dbg.getFontMetrics(phraseObj.getFont()).stringWidth(lose) /2));
		}
	}
*/    
    	
	
    // take a screen shot of the current display.  Before debug stats are reported.
    if (screenshot == true)
    {
		takeScreenShot(dbImage);   	
		screenshot = false;
    }

    
  //  reportStats(dbg);

    if (gameOver)
      gameOverMessage(dbg);

    if (showHelp)    // draw the help at the very front (if switched on)
      dbg.drawImage(helpIm, (PWIDTH-helpIm.getWidth())/2,
                          (PHEIGHT-helpIm.getHeight())/2, null);
  }  // end of gameRender()


  private void reportStats(Graphics g)
  // Report the number of hits, and time spent playing
  {
    if (!gameOver)    // stop incrementing the timer once the game is over
      timeSpentInGame =
          (int) ((System.nanoTime() - gameStartTime)/1000000000L);  // ns --> secs
    g.setColor(Color.red);
    g.setFont(msgsFont);
//  g.drawString("Hits: " + numHits + "/" + MAX_HITS, 15, 25);
    g.drawString("Time: " + timeSpentInGame + " secs", 15, 50);
    g.setColor(Color.black);
  }  // end of reportStats()


  private void gameOverMessage(Graphics g)
  // Center the game-over message in the panel.
  {
    String msg = "Game Over. Your score: " + score;

    int x = (PWIDTH - metrics.stringWidth(msg))/2;
    int y = (PHEIGHT - metrics.getHeight())/2;
    g.setColor(Color.black);
    g.setFont(msgsFont);
    g.drawString(msg, x, y);
  }  // end of gameOverMessage()


  private void paintScreen()
  // use active rendering to put the buffered image on-screen
  {
    Graphics g;
    try {
      g = this.getGraphics();
      if ((g != null) && (dbImage != null))
        g.drawImage(dbImage, 0, 0, null);
      // Sync the display on some systems.
      // (on Linux, this fixes event queue problems)
      Toolkit.getDefaultToolkit().sync();
      g.dispose();
    }
    catch (Exception e)
    { System.out.println("Graphics context error: " + e);  }
  } // end of paintScreen()

  private void takeScreenShot(Image dbi)
  {
  	String nowString = "";
	synchronized(this)  {
    	Date now = new Date();
    	SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd_HHmmssS" );
    	nowString = df.format(now);
	}
	
	File ssout = new File(nowString + ".png");
    try {
		ImageIO.write((RenderedImage) dbi, "png", ssout);
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }

  
  
  
}  // end of GamePanel class
