package net.lonestranger.game.object;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.LinkedList;
import java.util.List;

public class Phrase {

	String text;
	String phraseSetName;
		
	private int speed = 90; // pix per sec
	
	private double precise_locx = 800;
	private double precise_locy = 0;	
	
	int locx;
	int locy;
	
	int dx;
	int dy;

	int currentIndex;
	
	int failedKeys;
	
	FontMetrics fontMetrics;
	boolean isActive = true;
	boolean phraseDone = false;
	
	Font font;
	List<String> phraseSet;
	
	int phraseCount;
	
	
	public Phrase()
	{
		text = "This is a test String.";
		currentIndex = 0;
		
		phraseSetName = "Test Name";
		
		font = new Font("Courier New", Font.BOLD, 32);
//		font = new Font("Arial", Font.PLAIN, 12);
			
		locx = 800;//this.pwidth;
		locy = 600/2;
		
		dx = -1;
		dy = 0;
		
		failedKeys = 0;
		
		phraseSet = new LinkedList<String>();
		phraseCount = 10;
		
		
		//setNewPhrase();
		
	}
	
	public int checkNextLetter(char typedChar)
	{
		char nextLetter = '';
		
		if (currentIndex < text.length())
			nextLetter = text.charAt(currentIndex);
			
		if (nextLetter == typedChar)
		{	
			currentIndex++;
//			System.out.println(typedChar + "(" + (int)typedChar + ") == " + nextLetter  + "(" + (int)nextLetter + ")  -->ok");

			if(currentIndex >= text.length())
				phraseDone = true;

			return 0;
		}
		else
		{
//			System.out.println(typedChar + "(" + (int)typedChar + ") != " + nextLetter  + "(" + (int)nextLetter + ")  -->BAD");
			failedKeys++;
			return -1;
		}
		
	}
	public int checkPhraseDone()
	{
		if (phraseDone)
		{
			setNewPhrase();
			phraseCount--;
			
			//System.out.println("---------> " + phraseCount);
			if (phraseCount < 1)
				return 1;
			else
				return 0;
		}
		else
		{
			return -1;
		}
	}
	
	public int checkWallIntersection(int wallx)
	{
		int sw;
		
		if (fontMetrics != null)
			sw = fontMetrics.stringWidth(this.text.substring(0, currentIndex));
		else
			sw = 0;
		//System.out.println(wallx + " >= " + (this.locx + sw) + " = " + (wallx >= (this.locx + sw)));
		if (wallx >= (this.locx + sw))
		{
			return -1;
		}
		else
		{
			return 0;			
		}
	
	}
	
	public void draw(Graphics g)
	{
		fontMetrics = g.getFontMetrics(font);
	
		Font temp = g.getFont();
		g.setFont(this.font);

		String front = "";
		String back = "";
		
		// draw the first half of the phrase that has been typed
		if (currentIndex > 0)
		{
			front = this.text.substring(0,currentIndex);
			g.setColor(Color.lightGray);		
			g.drawString(front, this.locx, this.locy);
		}
		// draw the second half, that hasn't been typed.
		if (currentIndex <= text.length())
		{
			back = this.text.substring(currentIndex, text.length());
			int stringFrontWidth = fontMetrics.stringWidth(front);
			int stringBackWidth = fontMetrics.stringWidth(back);
			g.setColor(Color.black);		
			back = back.replace(" ", "_");  // does this make it easier?
			g.drawString(back, this.locx + stringFrontWidth, this.locy);
			g.setColor(Color.red);
			g.drawString("" + (char)171, this.locx + stringFrontWidth + stringBackWidth, this.locy);
		}

		g.setFont(temp);
		
		//g.drawString("(" + this.locx + "," + this.locy + ")", 10, 600 - 50);
		
	}

	public void update()
	{
	    if (isActive()) {
	        precise_locx -= ((double)speed)/30.0;
	        precise_locy += dy;
	        
	        locx = (int)precise_locx;
//	        if (isLooping)
//	          player.updateTick();  // update the player
	      }
	}
	
	public boolean isActive()
	{
		return isActive;
	}
	
	public void setNewPhrase()
	{
		currentIndex = 0;
		randomizePhrase();
		this.locx = 800;
		this.precise_locx = locx;
		this.phraseDone = false;
		
		this.failedKeys = 0;
		
	}
	
	public void randomizePhrase()
	{
//		List<String> list = new LinkedList<String>();
		
//		list.add("This is a test String");
//		list.add("Advancing Wall of Doom");
//		list.add("Ludum Dare");
//		list.add("uhfgood is here to harass us instead");
//		list.add("The Higher The Fewer");

		if ( (phraseSet != null) || (phraseSet.size() == 0) )
		{
			int num = (int) (Math.random() * phraseSet.size());
			this.text = phraseSet.get(num);
		}
		else
			this.text = "No Phrases in Set";
	}
	
	public void setPhraseSet(List<String> pl)
	{
		this.phraseSet = pl;
	}
	
	public Font getFont()
	{
		return font;
	}

	public int getFailedKeys() 
	{
		return failedKeys;
	}

	public int getPhraseLength() 
	{
		return this.text.length();
	}
	
	public void setPhraseSetName(String name)
	{
		this.phraseSetName = name;
	}

	public String getPhraseSetName() 
	{
		return this.phraseSetName;
	}

	public void setPhraseCount(int pc) {
		// TODO Auto-generated method stub
		this.phraseCount = pc;
	}
	
}
