package net.lonestranger.game.object;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

public class WordKeeper {

	Hashtable<String, List<String>> categories;
	
	private final static String PHRASE_DIR = "Phrases/";
	
	public WordKeeper(String phrs_file)
	{
	
		categories = new Hashtable<String, List<String>>();
		loadCategories(phrs_file);
		
		
		
	}  
	
	public void loadCategories(String fnm)
	{
		  
	    String phrFNm = PHRASE_DIR + fnm;
	    System.out.println("Reading file: " + phrFNm);
	    try 
	    {
			InputStream in = this.getClass().getResourceAsStream(phrFNm);
			//BufferedReader br = new BufferedReader( new InputStreamReader(in));
			BufferedReader br = new BufferedReader( new FileReader(phrFNm));
			String line;
			char ch;
			  
			while((line = br.readLine()) != null) 
			{
				line = line.trim();
				
				if (line.length() == 0)  // blank line
					continue;
				if (line.startsWith("//"))   // comment
					continue;
		        /*
				ch = Character.toLowerCase( line.charAt(0) );
		        if (ch == 'o')  // a single image
		        	;//getFileNameImage(line);
		        else if (ch == 'n')  // a numbered sequence of images
		        	;//getNumberedImages(line);
		        else if (ch == 's')  // an images strip
		        	;//getStripImages(line);
		        else if (ch == 'g')  // a group of images
		        	;//getGroupImages(line);
		        else
		        	System.out.println("Do not recognize line: " + line);
				*/
							
				categories.put(line, loadCategory(line));
				
			
			}  
			  
			br.close();
				
	    }
	    catch (IOException e) 
	    { 
	    	System.out.println("Error reading phrase declaration file: " + phrFNm);
	    	System.out.println("Error: " + e.getStackTrace());    
	    	System.exit(1);
	    }
	}

	private List<String> loadCategory(String filename) 
	{
		List<String> temp = new LinkedList<String>();
		
	    String phrFNm = PHRASE_DIR + filename;
	    System.out.println("Reading file: " + phrFNm);
	    try 
	    {		
			InputStream in = this.getClass().getResourceAsStream(phrFNm);
			//BufferedReader br = new BufferedReader( new InputStreamReader(in));
			BufferedReader br = new BufferedReader( new FileReader(phrFNm));
			String line;
			char ch;
			  
			while((line = br.readLine()) != null) 
			{
				line = line.trim();
				
				if (line.length() == 0)  // blank line
					continue;
				if (line.startsWith("//"))   // comment
					continue;

				//line = line.replace("'", "");
				//line = line.replace("`", "");
				line = line.replace("_", "");
			
				System.out.println(filename + ": " + line + " (" + temp.size() + ")");				
				
				if (line.trim() != "")
					temp.add(line);
				
				
			}  
			  
			br.close();
				
	    }
	    catch (IOException e) 
	    { 
	    	System.out.println("Error reading phrase file: " + phrFNm);
	    	System.out.println("Error: " + e.fillInStackTrace());    
	    	System.exit(1);
	    }
		
	    return temp;
	}

	public List<String> getRandomSet() {
		if ( (categories != null) && (categories.size() != 0) )
		{
			int num = (int) (Math.random() * categories.size());
			Set<String> key = categories.keySet();
			
			//System.out.println("new phrases: " + key.toArray()[num]);
		
			return categories.get(key.toArray()[num]);
		}
		else
			return null;
		

	}
	
	public void putRandomSet(Phrase phr)
	{
		String oldSet = phr.getPhraseSetName();
		
		if ( (categories != null) && (categories.size() != 0) )
		{
			do
			{
				int num = (int) (Math.random() * categories.size());
				Set<String> key = categories.keySet();
				phr.setPhraseSet(categories.get(key.toArray()[num]));
				phr.setPhraseSetName((String) key.toArray()[num]);
				System.out.println("new phrases: " + key.toArray()[num]);
				
			} while ((oldSet == phr.getPhraseSetName()));
		
			phr.setNewPhrase();
				
		}
	}
}
